@@ -4,11 +4,12 @@ use crate::calc_grammar_trait::CalcGrammarTrait;
 use crate::unary_operator::UnaryOperator;
 use id_tree::Tree;
 use log::trace;
-use parol_runtime::parser::errors::*;
+use parol_runtime::errors::*;
 use parol_runtime::parser::{ParseTreeStackEntry, ParseTreeType};
 use std::collections::BTreeMap;
 use std::convert::TryInto;
 use std::fmt::{Debug, Display, Error, Formatter};
+
 ///
 /// The value range for the supported calculations
 ///
@@ -83,7 +84,7 @@ impl Display for CalcGrammarItem {
 ///
 #[derive(Debug, Default)]
 pub struct CalcGrammar {
-    pub ast_stack: Vec<CalcGrammarItem>,
+    pub item_stack: Vec<CalcGrammarItem>,
     env: BTreeMap<String, DefinitionRange>,
 }
 
@@ -94,12 +95,12 @@ impl CalcGrammar {
 
     fn push(&mut self, item: CalcGrammarItem, context: &str) {
         trace!("push    {}: {}", context, item);
-        self.ast_stack.push(item)
+        self.item_stack.push(item)
     }
 
     fn pop(&mut self, context: &str) -> Option<CalcGrammarItem> {
-        if !self.ast_stack.is_empty() {
-            let item = self.ast_stack.pop();
+        if !self.item_stack.is_empty() {
+            let item = self.item_stack.pop();
             if let Some(ref item) = item {
                 trace!("pop     {}: {}", context, item);
             }
@@ -324,12 +325,12 @@ impl CalcGrammar {
 
     #[allow(dead_code)]
     // Use this function for debugging purposes:
-    // trace!("{}", self.trace_ast_stack(context));
-    fn trace_ast_stack(&self, context: &str) -> String {
+    // trace!("{}", self.trace_item_stack(context));
+    fn trace_item_stack(&self, context: &str) -> String {
         format!(
-            "Ast stack at {}:\n{}",
+            "Item stack at {}:\n{}",
             context,
-            self.ast_stack
+            self.item_stack
                 .iter()
                 .rev()
                 .map(|s| format!("  {}", s))
@@ -344,7 +345,7 @@ impl Display for CalcGrammar {
         writeln!(
             f,
             "Stack\n{}",
-            self.ast_stack
+            self.item_stack
                 .iter()
                 .rev()
                 .map(|e| format!("{}", e))
@@ -513,7 +514,7 @@ impl CalcGrammarTrait for CalcGrammar {
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "logical_or_13";
-        trace!("{}", self.trace_ast_stack(context));
+        trace!("{}", self.trace_item_stack(context));
         self.process_left_associative_operation_list(context)
     }
 
