@@ -5,6 +5,7 @@ use log::debug;
 use parol::analysis::follow_k;
 use parol::analysis::FirstCache;
 use parol::errors::*;
+use parol::generators::generate_terminal_names;
 use parol::{obtain_cfg_ext, MAX_K};
 use std::env;
 
@@ -39,8 +40,7 @@ fn run() -> Result<()> {
             bail!("Maximum lookahead is {}", MAX_K);
         }
 
-        let augmented_terminals = grammar_config.generate_augmented_terminals();
-        let terminals = augmented_terminals.to_vec();
+        let terminals = generate_terminal_names(&grammar_config);
         let first_cache = FirstCache::new();
         let follow_k = follow_k(&grammar_config, k, &first_cache);
         for (nt, fo) in follow_k.iter() {
